@@ -1,6 +1,10 @@
 use std::borrow::Cow;
+use std::collections::HashMap;
 use std::ops::RangeInclusive;
 
+pub mod acls;
+pub use acls::*;
+
 fn to_owned_cow<'a, T: ?Sized + ToOwned>(c: Cow<'a, T>) -> Cow<'static, T> {
     Cow::Owned(c.into_owned())
 }
@@ -43,6 +47,10 @@ pub enum Response<'a> {
     MailboxData(MailboxDatum<'a>),
     Quota(Quota<'a>),
     QuotaRoot(QuotaRoot<'a>),
+    Id(Option<HashMap<Cow<'a, str>, Cow<'a, str>>>),
+    Acl(Acl<'a>),
+    ListRights(ListRights<'a>),
+    MyRights(MyRights<'a>),
 }
 
 impl<'a> Response<'a> {
@@ -91,6 +99,14 @@ impl<'a> Response<'a> {
             Response::MailboxData(datum) => Response::MailboxData(datum.into_owned()),
             Response::Quota(quota) => Response::Quota(quota.into_owned()),
             Response::QuotaRoot(quota_root) => Response::QuotaRoot(quota_root.into_owned()),
+            Response::Id(map) => Response::Id(map.map(|m| {
+                m.into_iter()
+                    .map(|(k, v)| (to_owned_cow(k), to_owned_cow(v)))
+                    .collect()
+            })),
+            Response::Acl(acl_list) => Response::Acl(acl_list.into_owned()),
+            Response::ListRights(rights) => Response::ListRights(rights.into_owned()),
+            Response::MyRights(rights) => Response::MyRights(rights.into_owned()),
         }
     }
 }
